<?php
namespace Teknisa\Libs\Service;

use Teknisa\Libs\Util\Utilities;
use Zeedhi\Framework\Session\Session;
use Teknisa\Libs\Util\Login\UserData;
use Teknisa\Libs\Exception\Login as LoginException;

class Api {
    /** @var Session $sessionManager */
	protected $sessionManager;
    /** @var UserData $userData */
    private $userData;

    /**
     * @param Session $sessionManager
     * @param UserData $userData
     */
    public function __construct(Session $sessionManager, UserData $userData) {
        $this->sessionManager = $sessionManager;
        $this->userData = $userData;

        $config = json_decode(file_get_contents(__DIR__ . '/../../../../config/config.json'), true);
        $this->modulesPath = $config['modulesPath'];
    }

    /**
     * @param $nrOrg
     * @param $cdOperator
     */
    public function setSession($nrOrg, $cdOperator)
    {
        $this->sessionManager->set('NRORG', $nrOrg);
        $this->sessionManager->set('CDOPERADOR', $cdOperator);
    }

    /**
     * @param $user
     * @param $password
     * @return array
     */
    public function getUserData($user, $password) {
        return $this->userData->getUserData($user, $password);
    }

    /**
     * @param $email
     * @param $password
     * @param $identification
     * @return array
     */
    public function updatePassword($email, $password, $identification) {
        $this->userData->updatePassword($email, $password, $identification);
    }

    /**
     * @param $user
     * @param $password
     * @return array
     */
    public function validateUser($user, $password) {
        return $this->userData->validateUser($user, $password);
    }

    /**
     * @param bool $ignoreBowerNotFound
     * @return array
     */
    public function getModulesVersion($ignoreBowerNotFound = false) {
        $productBasePath = Utilities::getProductBasePath();
        $modules = @file_get_contents($productBasePath . $this->modulesPath);
        if (!$modules) {
            LoginException::modulesConfigNotFound();
        }

        $modules = json_decode($modules, true);
        $modulesVersion = $bowersNotFound = $modulesParametersNotFound = [];
        foreach($modules as $key => $module) {
            if(!isset($module['onlyControl'])) {
                if($key == Utilities::PRODUCT_MENU_NAME) {
                    $bowerFile = @file_get_contents($productBasePath . 'mobile/bower.json');
                } else {
                    $bowerFile = @file_get_contents($productBasePath . 'modules/' . $key . '/mobile/bower.json');
                    if(!$bowerFile) {
                        $bowerFile = @file_get_contents($productBasePath . 'modules/' . $key . '/productInfo.json');
                    }
                }
                if(!$bowerFile && !$ignoreBowerNotFound) {
                    $bowersNotFound[] = $key;
                } else if((!isset($module['maxVersion']) && $key != Utilities::PRODUCT_MENU_NAME) || !isset($module['devOpsId'])) {
                    $modulesParametersNotFound[] = $key;
                } else {
                    $bowerFile = json_decode($bowerFile, true);
                    if($key != Utilities::PRODUCT_MENU_NAME && isset($bowerFile['product'])) {
                        $bowerFile = $bowerFile['product'];
                    }
                    $modulesVersion[] = array(
                        'KEY'         => $key,
                        'NAME'        => isset($bowerFile['name']) ? $bowerFile['name'] : '',
                        'VERSION'     => $ignoreBowerNotFound && !$bowerFile ? '-' : $bowerFile['version'],
                        'MAX_VERSION' => $key != Utilities::PRODUCT_MENU_NAME ? $module['maxVersion'] : '*',
                        'DEVOPS_ID'   => $module['devOpsId']
                    );
                }
            }
        }

        if (!empty($bowersNotFound)) {
            LoginException::bowerConfigModuleNotFound(join(', ', $bowersNotFound));
        }
        if (!empty($modulesParametersNotFound)) {
            LoginException::moduleParametersNotFound(join('",<br/>"', $modulesParametersNotFound));
        }
        return $modulesVersion;
    }

    /**
     * @return string
     */
    public function generateModulesVersionFile() {
        $modulesVersion = $this->getModulesVersion();
        $modulesVersion = json_encode($modulesVersion);
        $generatedFile  = Utilities::getGeneratedFile();
        file_put_contents($generatedFile, $modulesVersion);
        $generatedFile = explode('/', $generatedFile);
        return array_pop($generatedFile);
    }

    public function downloadLoginAccessFile($fileName) {
        Utilities::downloadAndRemoveFile(Utilities::getGeneratedFileDir() . '/' . $fileName, $fileName);
    }

    public function setLastRequestTime($route) {
        if($this->sessionManager->isStarted() && $route != '/lib_validateExpiredSession') {
            $date = new \DateTime();
            $this->sessionManager->set('LAST_REQUEST_TIME', $date->getTimestamp());
        }
    }

}